@@ -1,16 +1,21 @@
 """Web mercator XYZ tile utilities"""
 
 from collections import namedtuple
-from collections import Sequence
 import math
+import sys
+
+if sys.version_info < (3,):
+    from collections import Sequence
+else:
+    from collections.abc import Sequence
 
 
-__version__ = '1.0.4'
+__version__ = '1.1.0'
 
 __all__ = [
     'Bbox', 'LngLat', 'LngLatBbox', 'Tile', 'bounding_tile', 'bounds',
     'children', 'feature', 'lnglat', 'parent', 'quadkey', 'quadkey_to_tile',
-    'tile', 'tiles', 'ul', 'xy_bounds']
+    'simplify', 'tile', 'tiles', 'ul', 'xy_bounds']
 
 
 Tile = namedtuple('Tile', ['x', 'y', 'z'])
@@ -61,6 +66,47 @@ class InvalidLatitudeError(MercantileError):
     """Raised when math errors occur beyond ~85 degrees N or S"""
 
 
+class InvalidZoomError(MercantileError):
+    """Raised when a zoom level is invalid"""
+
+
+class ParentTileError(MercantileError):
+    """Raised when a parent tile cannot be determined"""
+
+
+class QuadKeyError(MercantileError):
+    """Raised when errors occur in computing or parsing quad keys"""
+
+
+class TileArgParsingError(MercantileError):
+    """Raised when errors occur in parsing a function's tile arg(s)"""
+
+
+def _parse_tile_arg(*args):
+    """parse the *tile arg of module functions
+
+    Parameters
+    ----------
+    tile : Tile or sequence of int
+        May be be either an instance of Tile or 3 ints, X, Y, Z.
+
+    Returns
+    -------
+    Tile
+
+    Raises
+    ------
+    TileArgParsingError
+
+    """
+    if len(args) == 1:
+        args = args[0]
+    if len(args) == 3:
+        return Tile(*args)
+    else:
+        raise TileArgParsingError("the tile argument may have 1 or 3 values. Note that zoom is a keyword-only argument")
+
+
 def ul(*tile):
     """Returns the upper left longitude and latitude of a tile
 
@@ -81,9 +127,9 @@ def ul(*tile):
 
     >>> mercantile.ul(1, 1, 1)
     LngLat(lng=0.0, lat=0.0)
+
     """
-    if len(tile) == 1:
-        tile = tile[0]
+    tile = _parse_tile_arg(*tile)
     xtile, ytile, zoom = tile
     n = 2.0 ** zoom
     lon_deg = xtile / n * 360.0 - 180.0
@@ -103,9 +149,9 @@ def bounds(*tile):
     Returns
     -------
     LngLatBBox
+
     """
-    if len(tile) == 1:
-        tile = tile[0]
+    tile = _parse_tile_arg(*tile)
     xtile, ytile, zoom = tile
     a = ul(xtile, ytile, zoom)
     b = ul(xtile + 1, ytile + 1, zoom)
@@ -139,6 +185,7 @@ def xy(lng, lat, truncate=False):
     x, y : float
         y will be inf at the North Pole (lat >= 90) and -inf at the
         South Pole (lat <= -90).
+
     """
     if truncate:
         lng, lat = truncate_lnglat(lng, lat)
@@ -166,6 +213,7 @@ def lnglat(x, y, truncate=False):
     Returns
     -------
     LngLat
+
     """
     R2D = 180 / math.pi
     A = 6378137.0
@@ -188,9 +236,9 @@ def xy_bounds(*tile):
     Returns
     -------
     Bbox
+
     """
-    if len(tile) == 1:
-        tile = tile[0]
+    tile = _parse_tile_arg(*tile)
     xtile, ytile, zoom = tile
     left, top = xy(*ul(xtile, ytile, zoom))
     right, bottom = xy(*ul(xtile + 1, ytile + 1, zoom))
@@ -212,6 +260,7 @@ def tile(lng, lat, zoom, truncate=False):
     Returns
     -------
     Tile
+
     """
     if truncate:
         lng, lat = truncate_lnglat(lng, lat)
@@ -234,14 +283,15 @@ def quadkey(*tile):
 
     Parameters
     ----------
-    tile: Tile or sequence of int
+    tile : Tile or sequence of int
+        May be be either an instance of Tile or 3 ints, X, Y, Z.
 
     Returns
     -------
     str
+
     """
-    if len(tile) == 1:
-        tile = tile[0]
+    tile = _parse_tile_arg(*tile)
     xtile, ytile, zoom = tile
     qk = []
     for z in range(zoom, 0, -1):
@@ -261,12 +311,15 @@ def quadkey_to_tile(qk):
     Parameters
     ----------
     qk : str
-        The quadkey
+        A quadkey string.
 
     Returns
     -------
     Tile
+
     """
+    if len(qk) == 0:
+        return Tile(0, 0, 0)
     xtile, ytile = 0, 0
     for i, digit in enumerate(reversed(qk)):
         mask = 1 << i
@@ -278,7 +331,7 @@ def quadkey_to_tile(qk):
             xtile = xtile | mask
             ytile = ytile | mask
         elif digit != '0':
-            raise ValueError("Unexpected quadkey digit: %r", digit)
+            raise QuadKeyError("Unexpected quadkey digit: %r", digit)
     return Tile(xtile, ytile, i + 1)
 
 
@@ -297,6 +350,7 @@ def tiles(west, south, east, north, zooms, truncate=False):
     Yields
     ------
     Tile
+
     """
     if truncate:
         west, south = truncate_lnglat(west, south)
@@ -307,6 +361,7 @@ def tiles(west, south, east, north, zooms, truncate=False):
         bboxes = [bbox_west, bbox_east]
     else:
         bboxes = [(west, south, east, north)]
+
     for w, s, e, n in bboxes:
 
         # Clamp bounding values.
@@ -331,7 +386,7 @@ def tiles(west, south, east, north, zooms, truncate=False):
                     yield Tile(i, j, z)
 
 
-def parent(*tile):
+def parent(*tile, **kwargs):
     """Get the parent of a tile
 
     The parent is the tile of one zoom level lower that contains the
@@ -341,45 +396,157 @@ def parent(*tile):
     ----------
     tile : Tile or sequence of int
         May be be either an instance of Tile or 3 ints, X, Y, Z.
+    zoom : int, optional
+        Determines the *zoom* level of the returned parent tile.
+        This defaults to one lower than the tile (the immediate parent).
 
     Returns
     -------
     Tile
+
+    Examples
+    --------
+
+    >>> parent(Tile(0, 0, 2))
+    Tile(x=0, y=0, z=1)
+
+    >>> parent(Tile(0, 0, 2), zoom=0)
+    Tile(x=0, y=0, z=0)
+
     """
-    if len(tile) == 1:
-        tile = tile[0]
-    xtile, ytile, zoom = tile
-    # Algorithm ported directly from https://github.com/mapbox/tilebelt.
-    if xtile % 2 == 0 and ytile % 2 == 0:
-        return Tile(xtile // 2, ytile // 2, zoom - 1)
-    elif xtile % 2 == 0:
-        return Tile(xtile // 2, (ytile - 1) // 2, zoom - 1)
-    elif not xtile % 2 == 0 and ytile % 2 == 0:
-        return Tile((xtile - 1) // 2, ytile // 2, zoom - 1)
-    else:
-        return Tile((xtile - 1) // 2, (ytile - 1) // 2, zoom - 1)
+    tile = _parse_tile_arg(*tile)
+
+    # zoom is a keyword-only argument.
+    zoom = kwargs.get("zoom", None)
+
+    if zoom is not None and (tile[2] < zoom or zoom != int(zoom)):
+        raise InvalidZoomError(
+            "zoom must be an integer and less than that of the input tile")
+
+    x, y, z = tile
+    if x != int(x) or y != int(y) or z != int(z):
+        raise ParentTileError("the parent of a non-integer tile is undefined")
+
+    target_zoom = z - 1 if zoom is None else zoom
+
+    # Algorithm heavily inspired by https://github.com/mapbox/tilebelt.
+    return_tile = tile
+    while return_tile[2] > target_zoom:
+        xtile, ytile, ztile = return_tile
+        if xtile % 2 == 0 and ytile % 2 == 0:
+            return_tile = Tile(xtile // 2, ytile // 2, ztile - 1)
+        elif xtile % 2 == 0:
+            return_tile = Tile(xtile // 2, (ytile - 1) // 2, ztile - 1)
+        elif not xtile % 2 == 0 and ytile % 2 == 0:
+            return_tile = Tile((xtile - 1) // 2, ytile // 2, ztile - 1)
+        else:
+            return_tile = Tile((xtile - 1) // 2, (ytile - 1) // 2, ztile - 1)
+    return return_tile
 
 
-def children(*tile):
-    """Get the four children of a tile
+def children(*tile, **kwargs):
+    """Get the children of a tile
+
+    The children are ordered: top-left, top-right, bottom-right, bottom-left.
 
     Parameters
     ----------
     tile : Tile or sequence of int
         May be be either an instance of Tile or 3 ints, X, Y, Z.
+    zoom : int, optional
+        Returns all children at zoom *zoom*, in depth-first clockwise winding order.
+        If unspecified, returns the immediate (i.e. zoom + 1) children of the tile.
 
     Returns
     -------
     list
+
+    Examples
+    --------
+
+    >>> children(Tile(0, 0, 0))
+    [Tile(x=0, y=0, z=1), Tile(x=0, y=1, z=1), Tile(x=1, y=0, z=1), Tile(x=1, y=1, z=1)]
+
+    >>> children(Tile(0, 0, 0), zoom=2)
+    [Tile(x=0, y=0, z=2), Tile(x=0, y=1, z=2), Tile(x=0, y=2, z=2), Tile(x=0, y=3, z=2), ...]
+
     """
-    if len(tile) == 1:
-        tile = tile[0]
-    xtile, ytile, zoom = tile
-    return [
-        Tile(xtile * 2, ytile * 2, zoom + 1),
-        Tile(xtile * 2 + 1, ytile * 2, zoom + 1),
-        Tile(xtile * 2 + 1, ytile * 2 + 1, zoom + 1),
-        Tile(xtile * 2, ytile * 2 + 1, zoom + 1)]
+    tile = _parse_tile_arg(*tile)
+
+    # zoom is a keyword-only argument.
+    zoom = kwargs.get("zoom", None)
+
+    xtile, ytile, ztile = tile
+
+    if zoom is not None and (ztile > zoom or zoom != int(zoom)):
+        raise InvalidZoomError(
+            "zoom must be an integer and greater than that of the input tile")
+
+    target_zoom = zoom if zoom is not None else ztile + 1
+
+    tiles = [tile]
+    while tiles[0][2] < target_zoom:
+        xtile, ytile, ztile = tiles.pop(0)
+        tiles += [
+            Tile(xtile * 2, ytile * 2, ztile + 1),
+            Tile(xtile * 2 + 1, ytile * 2, ztile + 1),
+            Tile(xtile * 2 + 1, ytile * 2 + 1, ztile + 1),
+            Tile(xtile * 2, ytile * 2 + 1, ztile + 1)
+        ]
+    return tiles
+
+
+def simplify(tiles):
+    """Reduces the size of the tileset as much as possible by merging leaves into parents.
+
+    Parameters
+    ----------
+    tiles : Sequence of tiles to merge.
+
+    Returns
+    -------
+    list
+
+    """
+
+    def merge(merge_set):
+        """Checks to see if there are 4 tiles in merge_set which can be merged.
+        If there are, this merges them.
+        This returns a list of tiles, as well as a boolean indicating if any were merged.
+        By repeatedly applying merge, a tileset can be simplified.
+        """
+        upwards_merge = {}
+        for tile in merge_set:
+            tile_parent = parent(tile)
+            if tile_parent not in upwards_merge:
+                upwards_merge[tile_parent] = set()
+            upwards_merge[tile_parent] |= {tile}
+        current_tileset = []
+        changed = False
+        for supertile, children in upwards_merge.items():
+            if len(children) == 4:
+                current_tileset += [supertile]
+                changed = True
+            else:
+                current_tileset += list(children)
+        return current_tileset, changed
+
+    # Check to see if a tile and its parent both already exist.
+    # If so, discard the child (it's covered in the parent)
+    root_set = set()
+    for tile in tiles:
+        x, y, z = tile
+        supers = [parent(tile, zoom=i) for i in range(z + 1)]
+        for supertile in supers:
+            if supertile in root_set:
+                continue
+        root_set |= {tile}
+
+    # Repeatedly run merge until no further simplification is possible.
+    is_merging = True
+    while is_merging:
+        root_set, is_merging = merge(root_set)
+    return root_set
 
 
 def rshift(val, n):
@@ -400,6 +567,7 @@ def bounding_tile(*bbox, **kwds):
     Returns
     -------
     Tile
+
     """
     if len(bbox) == 2:
         bbox += bbox
@@ -460,6 +628,7 @@ def feature(
     Returns
     -------
     dict
+
     """
     west, south, east, north = bounds(tile)
     if projected == 'mercator':
